/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    d_apsgap.h 
 *
 * Purpose: Header file for MX motor driver for controlling an
 *          Advanced Photon Source undulator gap.
 */

#ifndef __D_APSGAP_H__
#define __D_APSGAP_H__

#include "mx_motor.h"

/* ===== MX APS undulator gap record data structures ===== */

typedef struct {
	int sector_number;
	int motor_subtype;
} MX_APS_GAP;

/* The subtypes of the aps_gap motor. */

#define MXT_APS_GAP		1
#define MXT_APS_KEV		2

/* Define all of the interface functions. */

MX_API mx_status_type mxd_aps_gap_initialize_type( long type );
MX_API mx_status_type mxd_aps_gap_create_record_structures(
						MX_RECORD *record );
MX_API mx_status_type mxd_aps_gap_finish_record_initialization(
						MX_RECORD *record );
MX_API mx_status_type mxd_aps_gap_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_aps_gap_print_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxd_aps_gap_read_parms_from_hardware(MX_RECORD *record);
MX_API mx_status_type mxd_aps_gap_write_parms_to_hardware(MX_RECORD *record);
MX_API mx_status_type mxd_aps_gap_open( MX_RECORD *record );
MX_API mx_status_type mxd_aps_gap_close( MX_RECORD *record );
MX_API mx_status_type mxd_aps_gap_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_positive_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_negative_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_aps_gap_find_home_position( MX_MOTOR *motor );

extern MX_RECORD_FUNCTION_LIST mxd_aps_gap_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_aps_gap_motor_function_list;

extern long mxd_aps_gap_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_aps_gap_record_field_def_ptr;

#define MXD_APS_GAP_STANDARD_FIELDS \
  {-1, -1, "sector_number", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_APS_GAP, sector_number), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "motor_subtype", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_APS_GAP, motor_subtype), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}

#endif /* __D_APSGAP_H__ */
