/*
 * Name:    d_epmtr.h 
 *
 * Purpose: Header file for MX motor driver for the EPICS APS-XFD developed
 *          motor record.
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_EPMTR_H__
#define __D_EPMTR_H__

#include "mx_motor.h"

/* ===== MX EPICS motor record data structures ===== */

typedef struct {
	char epics_record_name[41];
} MX_EPICS_MOTOR;

/* Define all of the interface functions. */

MX_API mx_status_type mxd_epics_motor_initialize_type( long type );
MX_API mx_status_type mxd_epics_motor_create_record_structures(
							MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_print_structure(
					FILE *file, MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_read_parms_from_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_write_parms_to_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_open( MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_close( MX_RECORD *record );
MX_API mx_status_type mxd_epics_motor_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_positive_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_negative_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_find_home_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_constant_velocity_move( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_get_parameter( MX_MOTOR *motor );
MX_API mx_status_type mxd_epics_motor_set_parameter( MX_MOTOR *motor );

extern MX_RECORD_FUNCTION_LIST mxd_epics_motor_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_epics_motor_motor_function_list;

extern long mxd_epics_motor_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_epics_motor_rfield_def_ptr;

#define MXD_EPICS_MOTOR_STANDARD_FIELDS \
  {-1, -1, "epics_record_name", MXFT_STRING, NULL, 1, {40}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_EPICS_MOTOR, epics_record_name), \
	{sizeof(char)}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}

#endif /* __D_EPMTR_H__ */

