/*
 * Name:    d_mmx000.h
 *
 * Purpose: Header file for MX drivers for the Newport MM3000, MM4000, and
 *          MM4005 series of motor controllers.
 *
 *-----------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __D_MMX000_H__
#define __D_MMX000_H__

#include "i_mmx000.h"

/* ============ Motor channels ============ */

typedef struct {
	MX_RECORD *mmx000_record;
	int axis_number;
} MX_MMX000_MOTOR;

MX_API mx_status_type mxd_mmx000_initialize_type( long type );
MX_API mx_status_type mxd_mmx000_create_record_structures(
						MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_finish_record_initialization(
						MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_print_structure( FILE *file,
						MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_read_parms_from_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_write_parms_to_hardware(
						MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_open( MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_close( MX_RECORD *record );
MX_API mx_status_type mxd_mmx000_resynchronize( MX_RECORD *record );

MX_API mx_status_type mxd_mmx000_motor_is_busy( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_move_absolute( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_get_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_set_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_soft_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_immediate_abort( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_positive_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_negative_limit_hit( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_find_home_position( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_constant_velocity_move( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_get_parameter( MX_MOTOR *motor );
MX_API mx_status_type mxd_mmx000_set_parameter( MX_MOTOR *motor );

extern MX_RECORD_FUNCTION_LIST mxd_mmx000_record_function_list;
extern MX_MOTOR_FUNCTION_LIST mxd_mmx000_motor_function_list;

extern long mxd_mm3000_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_mm3000_rfield_def_ptr;

extern long mxd_mm4000_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_mm4000_rfield_def_ptr;

#define MXD_MMX000_STANDARD_FIELDS \
  {-1, -1, "mmx000_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_MMX000_MOTOR, mmx000_record), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }, \
  \
  {-1, -1, "axis_number", MXFT_INT, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_MMX000_MOTOR, axis_number), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY) }

#endif /* __D_MMX000_H__ */
