/*
 * Name:    sq_struc.h
 *
 * Purpose: Header file for quick scans that use a Struck MCS via EPICS.
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __SQ_STRUC_H__
#define __SQ_STRUC_H__

#define MXS_SQ_MAX_MOTORS		8
#define MXS_SQ_NUM_STRUCK_SCALERS	16

typedef struct {
	char epics_record_name[80];

	double *motor_position_array[ MXS_SQ_MAX_MOTORS ];
	long *data_array[ MXS_SQ_NUM_STRUCK_SCALERS ];
} MX_STRUCK_QUICK_SCAN;

MX_API mx_status_type mxs_struck_quick_scan_initialize_type( long type );
MX_API mx_status_type mxs_struck_quick_scan_create_record_structures(
						MX_RECORD *record );
MX_API mx_status_type mxs_struck_quick_scan_finish_record_initialization(
						MX_RECORD *record );
MX_API mx_status_type mxs_struck_quick_scan_delete_record(
						MX_RECORD *record );
MX_API mx_status_type mxs_struck_quick_scan_dummy_function(
						MX_RECORD *record );

MX_API mx_status_type mxs_struck_quick_scan_prepare_for_scan_start(
						MX_SCAN *scan );
MX_API mx_status_type mxs_struck_quick_scan_execute_scan_body(
						MX_SCAN *scan );
MX_API mx_status_type mxs_struck_quick_scan_cleanup_after_scan_end(
						MX_SCAN *scan );

extern MX_RECORD_FUNCTION_LIST mxs_struck_quick_scan_record_function_list;
extern MX_SCAN_FUNCTION_LIST mxs_struck_quick_scan_scan_function_list;

extern long mxs_struck_quick_scan_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxs_struck_quick_scan_def_ptr;

#endif /* __SQ_STRUC_H__ */

