/*
 * Name:    mx_drivr.c
 *
 * Purpose: Describes the list of interfaces and devices included in
 *          this version of the program.
 */

#include <stdio.h>

#include "mx_drivr.h"

/* -- Include header files that define MX_XXX_FUNCTION_LIST structures. -- */

#include "mx_rs232.h"
#include "mx_camac.h"
#include "mx_ain.h"
#include "mx_aout.h"
#include "mx_din.h"
#include "mx_dout.h"
#include "mx_motor.h"
#include "mx_encod.h"
#include "mx_scalr.h"
#include "mx_timer.h"
#include "mx_scan.h"

/* Include the header files for all of the interfaces and devices. */

#include "i_tty.h"
#include "i_com.h"
#include "i_scamac.h"
#include "i_dsp600.h"
#include "i_pdi45.h"
#include "i_pdi40.h"
#include "d_ks3512.h"
#include "d_ks3112.h"
#include "d_ks3063.h"
#include "d_e500.h"
#include "d_smc24.h"
#include "d_panthr.h"
#include "d_ks3640.h"
#include "d_qs450.h"
#include "d_stimer.h"
#include "d_rtc018.h"
#include "d_pdi45.h"
#include "d_pdi40.h"
#include "s_scaler.h"
#include "s_motor.h"
#include "s_2theta.h"

/* -- Define lists that relate types to classes and to function lists. -- */

MX_TYPE_LIST_ENTRY mx_record_class_list[] = {
{MXR_INTERFACE, "int", 0, NULL, NULL},
{MXR_DEVICE,    "dev", 0, NULL, NULL},
{MXR_SCAN,      "scan", 0, NULL, NULL},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_interface_class_list[] = {

  /* First list generic type interfaces. */

{MXI_RS232, "rs232", MXR_INTERFACE, &mx_rs232_generic_function_list, NULL},

  /* Then we follow with multi-io board type interfaces. */

{MXI_PDI40, "pdi40", MXR_INTERFACE, &mxi_pdi40_generic_function_list, NULL},

{0, "", 0, NULL, NULL}
};


MX_TYPE_LIST_ENTRY mx_interface_type_list[] = {

{MXI_232_COM,      "com",        MXI_RS232,
		&mxi_com_generic_function_list,
		&mxi_com_rs232_function_list},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_device_class_list[] = {
{MXC_ANALOG_INPUT,   "analog_input",   MXR_DEVICE,
				&mx_analog_input_generic_function_list,
				NULL},
{MXC_ANALOG_OUTPUT,  "analog_output",  MXR_DEVICE,
				&mx_analog_output_generic_function_list,
				NULL},
{MXC_DIGITAL_INPUT,  "digital_input",  MXR_DEVICE,
				&mx_digital_input_generic_function_list,
				NULL},
{MXC_DIGITAL_OUTPUT, "digital_output", MXR_DEVICE,
				&mx_digital_output_generic_function_list,
				NULL},
{MXC_MOTOR,          "motor",          MXR_DEVICE,
				&mx_motor_generic_function_list,
				NULL},
{MXC_ENCODER,        "encoder",        MXR_DEVICE,
				&mx_encoder_generic_function_list,
				NULL},
{MXC_SCALER,         "scaler",         MXR_DEVICE,
				&mx_scaler_generic_function_list,
				NULL},
{MXC_TIMER,          "timer",          MXR_DEVICE,
				&mx_timer_generic_function_list,
				NULL},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_device_type_list[] = {
{MXT_MTR_PANTHER,	"panther",	MXC_MOTOR,
		&mxd_panther_generic_function_list,
		&mxd_panther_motor_function_list},
{MXT_MTR_PDI40,         "pdi40_motor",  MXC_MOTOR,
		&mxd_pdi40motor_generic_function_list,
		&mxd_pdi40motor_motor_function_list},

{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_scan_class_list[] = {
{MXS_SCAN_1D, "scan_1d", MXR_SCAN, NULL, NULL},
{0, "", 0, NULL, NULL}
};

MX_TYPE_LIST_ENTRY mx_scan_type_list[] = {
{MXS_1D_SCALER, "scaler", MXS_SCAN_1D,
		&mxs_scan_scaler_generic_function_list,
		&mxs_scan_scaler_scan_1d_function_list},
{MXS_1D_MOTOR,  "motor",  MXS_SCAN_1D,
		&mxs_scan_motor_generic_function_list,
		&mxs_scan_motor_scan_1d_function_list},
{MXS_1D_2THETA, "2theta", MXS_SCAN_1D,
		&mxs_scan_2theta_generic_function_list,
		&mxs_scan_2theta_scan_1d_function_list},
{0, "", 0, NULL, NULL}
};

